@@ -2,6 +2,7 @@
     SPDX-FileCopyrightText: 2013 Kevin Ottens <ervin+bluesystems@kde.org>
     SPDX-FileCopyrightText: 2013 Aleix Pol Gonzalez <aleixpol@blue-systems.com>
     SPDX-FileCopyrightText: 2014 Lukáš Tinkl <ltinkl@redhat.com>
+    SPDX-FileCopyrightText: 2022 Harald Sitter <sitter@kde.org>
 
     SPDX-License-Identifier: LGPL-2.0-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 */
@@ -27,8 +28,15 @@
 #include <QtQuickControls2/QQuickStyle>
 
 #include <KIO/Global>
+#include <KIO/JobUiDelegate>
+#include <KIO/JobUiDelegateFactory>
+#include <KIO/OpenWithHandlerInterface>
+#include <KJobWidgets>
 #include <KLocalizedString>
 #include <KStandardGuiItem>
+#include <KWayland/Client/connection_thread.h>
+#include <KWayland/Client/registry.h>
+#include <KWayland/Client/xdgforeign.h>
 #include <KWindowSystem>
 #include <kiconengine.h>
 #include <kiconloader.h>
@@ -57,18 +65,203 @@ static bool isDBusGlobalMenuAvailable()
     return dbusGlobalMenuAvailable;
 }
 
+static QString desktopPortalService()
+{
+    return QStringLiteral("org.freedesktop.impl.portal.desktop.kde");
+}
+
+static QString desktopPortalPath()
+{
+    return QStringLiteral("/org/freedesktop/portal/desktop");
+}
+
+class XdgWindowExporter : public QObject
+{
+    Q_OBJECT
+public:
+    using QObject::QObject;
+    virtual void run(QWidget *widget) = 0;
+Q_SIGNALS:
+    void exported(const QString &id);
+};
+
+class XdgWindowExporterWayland : public XdgWindowExporter
+{
+public:
+    using XdgWindowExporter::XdgWindowExporter;
+
+    void run(QWidget *widget) override
+    {
+        Q_ASSERT(widget);
+
+        auto connection = KWayland::Client::ConnectionThread::fromApplication(QGuiApplication::instance());
+        if (!connection) {
+            Q_EMIT exported({});
+            return;
+        }
+
+        auto registry = new KWayland::Client::Registry(this);
+        QPointer<QWidget> maybeWidget(widget);
+        connect(registry, &KWayland::Client::Registry::exporterUnstableV2Announced, this, [this, registry, maybeWidget](quint32 name, quint32 version) {
+            auto exporter = registry->createXdgExporter(name, std::min(version, quint32(1)), this);
+            if (!maybeWidget) {
+                qWarning() << "widget was invalid, not exporting any window as transient parent";
+                Q_EMIT exported({});
+                return;
+            }
+            auto surface = KWayland::Client::Surface::fromWindow(maybeWidget->windowHandle());
+            if (!surface) {
+                qWarning() << "wayland surface was unexpectedly null, not exporting any window as transient parent";
+                Q_EMIT exported({});
+                return;
+            }
+            auto xdgExported = exporter->exportTopLevel(surface, this);
+            Q_EMIT exported(QLatin1String("wayland:") + xdgExported->handle());
+        });
+
+        registry->create(connection);
+        registry->setup();
+    }
+};
+
+class XdgWindowExporterX11 : public XdgWindowExporter
+{
+public:
+    using XdgWindowExporter::XdgWindowExporter;
+
+    void run(QWidget *widget) override
+    {
+        Q_ASSERT(widget);
+        Q_EMIT exported(QLatin1String("x11:") + QString::number(widget->winId(), 16));
+    }
+};
+
+class KIOOpenWith : public KIO::OpenWithHandlerInterface
+{
+    Q_OBJECT
+public:
+    explicit KIOOpenWith(QWidget *parentWidget, QObject *parent = nullptr)
+        : KIO::OpenWithHandlerInterface(parent)
+        , m_parentWidget(parentWidget)
+    {
+    }
+
+    void promptUserForApplication(KJob *job, const QList<QUrl> &urls, const QString &mimeType) override
+    {
+        Q_UNUSED(mimeType);
+
+        QWidget *widget = nullptr;
+        if (job) {
+            widget = KJobWidgets::window(job);
+        }
+
+        if (!widget) {
+            widget = m_parentWidget;
+        }
+
+        if (!widget) {
+            promptInternal({}, urls);
+            return;
+        }
+
+        widget->winId(); // ensure we have a handle so we can export a window (without this windowHandle() may be null)
+
+        XdgWindowExporter *exporter = nullptr;
+        switch (KWindowSystem::platform()) {
+        case KWindowSystem::Platform::X11:
+            exporter = new XdgWindowExporterX11(this);
+            break;
+        case KWindowSystem::Platform::Wayland:
+            exporter = new XdgWindowExporterWayland(this);
+            break;
+        case KWindowSystem::Platform::Unknown:
+            break;
+        }
+        if (exporter) {
+            connect(exporter, &XdgWindowExporter::exported, this, [this, urls, exporter](const QString &windowId) {
+                exporter->deleteLater();
+                promptInternal(windowId, urls);
+            });
+            exporter->run(widget);
+        } else {
+            promptInternal({}, urls);
+        }
+    }
+
+    void promptInternal(const QString &windowId, const QList<QUrl> &urls)
+    {
+        QDBusMessage message = QDBusMessage::createMethodCall(desktopPortalService(),
+                                                              desktopPortalPath(),
+                                                              QStringLiteral("org.freedesktop.impl.portal.AppChooser"),
+                                                              QStringLiteral("ChooseApplicationPrivate"));
+
+        QStringList urlStrings;
+        for (const auto &url : urls) {
+            urlStrings << url.toString();
+        }
+        message << windowId << urlStrings << QVariantMap{{QStringLiteral("ask"), true}};
+
+        QDBusPendingCall pendingCall = QDBusConnection::sessionBus().asyncCall(message, std::numeric_limits<int>::max());
+        auto watcher = new QDBusPendingCallWatcher(pendingCall, this);
+        connect(watcher, &QDBusPendingCallWatcher::finished, this, [this](QDBusPendingCallWatcher *watcher) {
+            watcher->deleteLater();
+
+            QDBusPendingReply<uint, QVariantMap> reply = *watcher;
+            if (reply.isError()) {
+                qWarning() << "Couldn't get reply";
+                qWarning() << "Error: " << reply.error().message();
+                Q_EMIT canceled();
+            } else {
+                if (reply.argumentAt<0>() == 0) {
+                    Q_EMIT serviceSelected(KService::serviceByDesktopName(reply.argumentAt<1>().value(QStringLiteral("choice")).toString()));
+                } else {
+                    Q_EMIT canceled();
+                }
+            }
+        });
+    }
+
+private:
+    QWidget *const m_parentWidget;
+};
+
+class KIOUiDelegate : public KIO::JobUiDelegate
+{
+public:
+    explicit KIOUiDelegate(KJobUiDelegate::Flags flags = AutoHandlingDisabled, QWidget *window = nullptr)
+        : KIO::JobUiDelegate(KIO::JobUiDelegate::Version::V2, flags, window, {new KIOOpenWith(window, nullptr)})
+    {
+    }
+};
+
+class KIOUiFactory : public KIO::JobUiDelegateFactoryV2
+{
+public:
+    KIOUiFactory() = default;
+
+    KJobUiDelegate *createDelegate() const override
+    {
+        return new KIOUiDelegate;
+    }
+
+    KJobUiDelegate *createDelegate(KJobUiDelegate::Flags flags, QWidget *window) const override
+    {
+        return new KIOUiDelegate(flags, window);
+    }
+};
+
 KdePlatformTheme::KdePlatformTheme()
 {
     loadSettings();
 
     // explicitly not KWindowSystem::isPlatformWayland to not include the kwin process
     if (QGuiApplication::platformName() == QLatin1String("wayland")) {
-        m_kwaylandIntegration.reset(new KWaylandIntegration());
+        m_kwaylandIntegration.reset(new KWaylandIntegration(this));
     }
 
 #if HAVE_X11
     if (KWindowSystem::isPlatformX11()) {
-        m_x11Integration.reset(new X11Integration());
+        m_x11Integration.reset(new X11Integration(this));
         m_x11Integration->init();
     }
 #endif
@@ -85,6 +278,12 @@ KdePlatformTheme::KdePlatformTheme()
 
     QCoreApplication::setAttribute(Qt::AA_DontUseNativeMenuBar, false);
     setQtQuickControlsTheme();
+
+    static KIOUiFactory factory;
+    KIO::setDefaultJobUiDelegateFactoryV2(&factory);
+
+    static KIOUiDelegate delegateExtension;
+    KIO::setDefaultJobUiDelegateExtension(&delegateExtension);
 }
 
 KdePlatformTheme::~KdePlatformTheme()
@@ -289,11 +488,11 @@ QString KdePlatformTheme::standardButtonText(int button) const
     case QPlatformDialogHelper::Open:
         return KStandardGuiItem::open().text();
     case QPlatformDialogHelper::Yes:
-        return KStandardGuiItem::yes().text();
+        return i18nc("@action:button", "&Yes");
     case QPlatformDialogHelper::YesToAll:
         return i18nc("@action:button", "Yes to All");
     case QPlatformDialogHelper::No:
-        return KStandardGuiItem::no().text();
+        return i18nc("@action:button", "&No");
     case QPlatformDialogHelper::NoToAll:
         return i18nc("@action:button", "No to All");
     case QPlatformDialogHelper::Abort:
@@ -346,36 +545,19 @@ QPlatformSystemTrayIcon *KdePlatformTheme::createPlatformSystemTrayIcon() const
 QPlatformMenuBar *KdePlatformTheme::createPlatformMenuBar() const
 {
     if (isDBusGlobalMenuAvailable()) {
-        auto *menu = new QDBusMenuBar();
+#ifndef KF6_TODO_DBUS_MENUBAR
+        auto *menu = new QDBusMenuBar(const_cast<KdePlatformTheme *>(this));
 
         QObject::connect(menu, &QDBusMenuBar::windowChanged, menu, [this, menu](QWindow *newWindow, QWindow *oldWindow) {
             const QString &serviceName = QDBusConnection::sessionBus().baseService();
             const QString &objectPath = menu->objectPath();
 
-            if (m_x11Integration) {
-                if (oldWindow) {
-                    m_x11Integration->setWindowProperty(oldWindow, s_x11AppMenuServiceNamePropertyName, {});
-                    m_x11Integration->setWindowProperty(oldWindow, s_x11AppMenuObjectPathPropertyName, {});
-                }
-
-                if (newWindow) {
-                    m_x11Integration->setWindowProperty(newWindow, s_x11AppMenuServiceNamePropertyName, serviceName.toUtf8());
-                    m_x11Integration->setWindowProperty(newWindow, s_x11AppMenuObjectPathPropertyName, objectPath.toUtf8());
-                }
-            }
-
-            if (m_kwaylandIntegration) {
-                if (oldWindow) {
-                    m_kwaylandIntegration->setAppMenu(oldWindow, QString(), QString());
-                }
-
-                if (newWindow) {
-                    m_kwaylandIntegration->setAppMenu(newWindow, serviceName, objectPath);
-                }
-            }
+            setMenuBarForWindow(oldWindow, {}, {});
+            setMenuBarForWindow(newWindow, serviceName, objectPath);
         });
 
         return menu;
+#endif
     }
 
     return nullptr;
@@ -404,3 +586,71 @@ bool KdePlatformTheme::useXdgDesktopPortal()
     static bool usePortal = qEnvironmentVariableIntValue("PLASMA_INTEGRATION_USE_PORTAL") == 1;
     return usePortal;
 }
+
+inline bool windowRelevantForGlobalMenu(QWindow *window)
+{
+    return !(window->type() & Qt::WindowType::Popup);
+}
+
+void KdePlatformTheme::globalMenuBarExistsNow()
+{
+#ifndef KF6_TODO_DBUS_MENUBAR
+    const QString &serviceName = QDBusConnection::sessionBus().baseService();
+    const QString &objectPath = QDBusMenuBar::globalMenuBar()->objectPath();
+
+    for (auto *window : qApp->topLevelWindows()) {
+        if (QDBusMenuBar::menuBarForWindow(window))
+            continue;
+        if (!windowRelevantForGlobalMenu(window))
+            return;
+
+        setMenuBarForWindow(window, serviceName, objectPath);
+    }
+#endif
+}
+
+void KdePlatformTheme::windowCreated(QWindow *window)
+{
+#ifndef KF6_TODO_DBUS_MENUBAR
+    if (!QDBusMenuBar::globalMenuBar())
+        return;
+
+    if (QDBusMenuBar::menuBarForWindow(window))
+        return;
+
+    const QString &serviceName = QDBusConnection::sessionBus().baseService();
+    const QString &objectPath = QDBusMenuBar::globalMenuBar()->objectPath();
+
+    setMenuBarForWindow(window, serviceName, objectPath);
+#endif
+}
+
+void KdePlatformTheme::globalMenuBarNoLongerExists()
+{
+#ifndef KF6_TODO_DBUS_MENUBAR
+    for (auto *window : qApp->topLevelWindows()) {
+        if (QDBusMenuBar::menuBarForWindow(window))
+            continue;
+        if (!windowRelevantForGlobalMenu(window))
+            return;
+
+        setMenuBarForWindow(window, {}, {});
+    }
+#endif
+}
+
+void KdePlatformTheme::setMenuBarForWindow(QWindow *window, const QString &serviceName, const QString &objectPath) const
+{
+    if (!window)
+        return;
+
+    if (m_x11Integration) {
+        m_x11Integration->setWindowProperty(window, s_x11AppMenuServiceNamePropertyName, serviceName.toUtf8());
+        m_x11Integration->setWindowProperty(window, s_x11AppMenuObjectPathPropertyName, objectPath.toUtf8());
+    }
+
+    if (m_kwaylandIntegration) {
+        m_kwaylandIntegration->setAppMenu(window, serviceName, objectPath);
+    }
+}
+#include "kdeplatformtheme.moc"
