@@ -20,34 +20,35 @@
 #include <QStringList>
 #include <QUrl>
 
+#include <KActionCollection>
+#include <KAuthorized>
+#include <KConfig>
+#include <KConfigGroup>
+#include <KFileItemDelegate>
 #include <KFileUtils>
+#include <KFileWidget>
+#include <KHistoryComboBox>
+#include <KIO/CopyJob>
+#include <KIO/DeleteJob>
+#include <KIO/DeleteOrTrashJob>
+#include <KIO/Job>
+#include <KIO/JobUiDelegate>
+#include <KIO/MkdirJob>
+#include <KJobWidgets>
+#include <KLocalizedString>
+#include <KMessageBox>
+#include <KPropertiesDialog>
+#include <KRecentDirs>
+#include <KService>
+#include <KSharedConfig>
 #include <KStandardShortcut>
-#include <kactioncollection.h>
-#include <kauthorized.h>
-#include <kconfig.h>
-#include <kconfiggroup.h>
-#include <kfileitemdelegate.h>
-#include <kfilewidget.h>
-#include <khistorycombobox.h>
-#include <kio/copyjob.h>
-#include <kio/deletejob.h>
-#include <kio/job.h>
-#include <kio/jobuidelegate.h>
-#include <kio/mkdirjob.h>
-#include <kjobwidgets.h>
-#include <klocalizedstring.h>
-#include <kmessagebox.h>
-#include <kpropertiesdialog.h>
-#include <krecentdirs.h>
-#include <kservice.h>
-#include <ksharedconfig.h>
-#include <ktoggleaction.h>
-#include <kurlcompletion.h>
-#include <kurlpixmapprovider.h>
+#include <KToggleAction>
+#include <KUrlCompletion>
+#include <KUrlPixmapProvider>
 
 #include "kfiletreeview_p.h"
-#include <kfileplacesmodel.h>
-#include <kfileplacesview.h>
+#include <KFilePlacesModel>
+#include <KFilePlacesView>
 // ### add mutator for treeview!
 
 class KDirSelectDialog::Private
@@ -238,23 +239,17 @@ void KDirSelectDialog::Private::slotExpand(const QModelIndex &index)
 void KDirSelectDialog::Private::slotMoveToTrash()
 {
     const QUrl url = m_treeView->selectedUrl();
-    KIO::JobUiDelegate job;
-    if (job.askDeleteConfirmation(QList<QUrl>() << url, KIO::JobUiDelegate::Trash, KIO::JobUiDelegate::DefaultConfirmation)) {
-        KIO::CopyJob *copyJob = KIO::trash(url);
-        KJobWidgets::setWindow(copyJob, m_parent);
-        copyJob->uiDelegate()->setAutoErrorHandlingEnabled(true);
-    }
+    using Iface = KIO::AskUserActionInterface;
+    auto *trashJob = new KIO::DeleteOrTrashJob({url}, Iface::Trash, Iface::DefaultConfirmation, m_parent);
+    trashJob->start();
 }
 
 void KDirSelectDialog::Private::slotDelete()
 {
     const QUrl url = m_treeView->selectedUrl();
-    KIO::JobUiDelegate job;
-    if (job.askDeleteConfirmation(QList<QUrl>() << url, KIO::JobUiDelegate::Delete, KIO::JobUiDelegate::DefaultConfirmation)) {
-        KIO::DeleteJob *deleteJob = KIO::del(url);
-        KJobWidgets::setWindow(deleteJob, m_parent);
-        deleteJob->uiDelegate()->setAutoErrorHandlingEnabled(true);
-    }
+    using Iface = KIO::AskUserActionInterface;
+    auto *deleteJob = new KIO::DeleteOrTrashJob({url}, Iface::Delete, Iface::DefaultConfirmation, m_parent);
+    deleteJob->start();
 }
 
 void KDirSelectDialog::Private::slotProperties()
